# -*- coding: utf-8 -*-
# (c) 2021, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import os
import shutil
import sys
import tempfile
from contextlib import contextmanager

import yaml


class AndeboxAction:
    name = None
    help = None
    args = []  # of dict(names=[]: specs={})
    coll_copy_exclusion = ('.git', '.tox', '.venv', '.virtualvenv', 'venv', 'virtualenv')

    @staticmethod
    def copy_collection(coll_dir):
        # copy files to tmp ansible coll dir
        with os.scandir() as it:
            for entry in it:
                if any(entry.name.startswith(x) for x in AndeboxAction.coll_copy_exclusion):
                    continue
                if entry.is_dir():
                    shutil.copytree(entry.name, os.path.join(coll_dir, entry.name))
                else:
                    shutil.copy(entry.name, os.path.join(coll_dir, entry.name))

    @staticmethod
    def binary_path(venv, bin):
        _list = ([venv, "bin"] if venv else []) + [bin]
        return os.path.join(*_list)

    @contextmanager
    def ansible_collection_tree(self, namespace, collection, keep=False):
        top_dir = ""
        coll_dir = ""
        try:
            top_dir = tempfile.mkdtemp(prefix="andebox.")
            coll_dir = os.path.join(top_dir, "ansible_collections", namespace, collection)
            os.makedirs(coll_dir)
            print("{0:10} = {1}.{2}".format("collection", namespace, collection), file=sys.stderr)
            print("{0:10} = {1}".format("directory", coll_dir), file=sys.stderr)

            self.copy_collection(coll_dir)
            os.putenv('ANSIBLE_COLLECTIONS_PATH', ':'.join([top_dir] + os.environ.get('ANSIBLE_COLLECTIONS_PATH', '').split(':')))
            yield coll_dir

        finally:
            if not keep:
                print('Removing temporary directory: {0}'.format(coll_dir))
                shutil.rmtree(top_dir)
            else:
                print('Keeping temporary directory: {0}'.format(coll_dir))

    def make_parser(self, subparser):
        action_parser = subparser.add_parser(self.name, help=self.help)
        for arg in self.args:
            action_parser.add_argument(*arg['names'], **arg['specs'])
        return action_parser

    def run(self, args):
        raise NotImplementedError()

    @staticmethod
    def read_coll_meta():
        with open("galaxy.yml") as galaxy_meta:
            meta = yaml.load(galaxy_meta, Loader=yaml.BaseLoader)
        return meta['namespace'], meta['name'], meta['version']

    def determine_collection(self, coll_arg):
        if coll_arg:
            coll_split = coll_arg.split('.')
            return '.'.join(coll_split[:-1]), coll_split[-1]
        return self.read_coll_meta()[:2]

    @staticmethod
    def copy_exclude_lines(src, dest, exclusion_filenames):
        with open(src, "r") as src_file, open(dest, "w") as dest_file:
            for line in src_file.readlines():
                if not any(line.startswith(f) for f in exclusion_filenames):
                    dest_file.write(line)

    def __str__(self):
        return "<AndeboxAction: {name}>".format(name=self.name)



# -*- coding: utf-8 -*-
# (c) 2021, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class AndeboxException(Exception):
    pass



# -*- coding: utf-8 -*-
# (c) 2021, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import os
import subprocess

import yaml

from andeboxlib.actions.base import AndeboxAction
from andeboxlib.exceptions import AndeboxException


class AnsibleTestError(AndeboxException):
    pass


class AnsibleTestAction(AndeboxAction):
    name = "test"
    help = "runs ansible-test in a temporary environment"
    args = [
        dict(names=("--keep", "-k"),
             specs=dict(action="store_true", help="Keep temporary directory after execution")),
        dict(names=("--exclude-from-ignore", "-efi", "-ei"),
             specs=dict(action="store_true", help="Matching lines in ignore files will be filtered out")),
        dict(names=("--requirements", "-R"),
             specs=dict(action="store_true",
                        help="Install integration_tests_dependencies from tests/requirements.yml prior")),
        dict(names=("--venv", "-V"),
             specs=dict(help="""path to the virtual environment where andebox and ansible are installed""")),
        dict(names=("ansible_test_params", ),
             specs=dict(nargs="+")),
    ]

    def make_parser(self, subparser):
        action_parser = super().make_parser(subparser)
        action_parser.epilog = "Notice the use of '--' to delimit andebox's options from ansible-test's"
        action_parser.usage = "%(prog)s usage: andebox test [-h] [--keep] -- [ansible_test_params ...]"

    def run(self, args):
        namespace, collection = self.determine_collection(args.collection)
        if args.requirements:
            reqs = self.obtain_reqs()
        with self.ansible_collection_tree(namespace, collection, args.keep) as collection_dir:
            if args.requirements:
                self.install_requirements(reqs, args.venv)
            if args.exclude_from_ignore:
                self.exclude_from_ignore(args.exclude_from_ignore, args.ansible_test_params, collection_dir)
            rc = subprocess.call([self.binary_path(args.venv, "ansible-test")] + args.ansible_test_params, cwd=collection_dir)

            if rc != 0:
                raise AnsibleTestError("Error running ansible-test (rc={0})".format(rc))

    def exclude_from_ignore(self, exclude_from_ignore, ansible_test_params, coll_dir):
        files = [f for f in ansible_test_params if os.path.isfile(f)]
        print("Excluding from ignore files: {files}".format(files=files))
        if exclude_from_ignore:
            src_dir = os.path.join(os.getcwd(), 'tests', 'sanity')
            dest_dir = os.path.join(coll_dir, 'tests', 'sanity')
            with os.scandir(src_dir) as ts_dir:
                for ts_entry in ts_dir:
                    if ts_entry.name.startswith('ignore-') and ts_entry.name.endswith('.txt'):
                        self.copy_exclude_lines(os.path.join(src_dir, ts_entry.name),
                                                os.path.join(dest_dir, ts_entry.name),
                                                files)

    @staticmethod
    def obtain_reqs():
        reqs_path = os.path.join('.', 'tests', 'requirements.yml')
        with open(reqs_path) as yaml_file:
            reqs_yaml = yaml.load(yaml_file, Loader=yaml.BaseLoader)
        return reqs_yaml['integration_tests_dependencies']

    @staticmethod
    def install_requirements(reqs, venv):
        rc = subprocess.call([AnsibleTestAction.binary_path(venv, "ansible-galaxy"), "collection", "install"] + reqs)

        if rc != 0:
            raise AnsibleTestError("Error installing dependencies (rc={0})".format(rc))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

requirements = [line.strip() for line in open('requirements.txt').readlines()]

setup(name='andebox',
      version='0.31',
      scripts=['andebox'],
      packages=find_packages(),
      install_requires=requirements,
      )



# -*- coding: utf-8 -*-
# (c) 2021, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import os

from fabric.connection import Connection

from andeboxlib.actions.base import AndeboxAction
from andeboxlib.exceptions import AndeboxException


class VagrantError(AndeboxException):
    pass


class VagrantAction(AndeboxAction):
    name = "vagrant"
    help = "runs 'andebox test -- integration' within a VM managed with vagrant"
    args = [
        dict(names=("--name", "-n"),
             specs=dict(help="""name of the vagrant VM (default: "default")""")),
        dict(names=("--sudo", "-s"),
             specs=dict(action="store_true",
                        help="""use sudo to run andebox""")),
        dict(names=("--venv", "-V"),
             specs=dict(help="""path to the virtual environment where andebox and ansible are installed (default: "/venv")""")),
        dict(names=("andebox_params", ),
             specs=dict(nargs="+")),
    ]
    default_name = "default"
    default_venv = "/venv"

    def make_parser(self, subparser):
        action_parser = super().make_parser(subparser)
        action_parser.epilog = "Notice the use of '--' to delimit the vagrant command from the one running inside the VM"
        action_parser.usage = "%(prog)s usage: andebox vagrant [-h] [-n name] -- <andebox-cmd> [andebox-cmd-opts [-- test-params]]"

    def run(self, args):
        import vagrant

        if not os.path.exists("Vagrantfile"):
            raise VagrantError("Missing Vagrantfile in the current directory")

        # argparse does not seem to honour defaults in subparsers, so making do with these
        machine_name = args.name or self.default_name
        venv = args.venv or self.default_venv

        print(f"== SETUP vagrant VM: {machine_name} {'=' * 80}")
        v = vagrant.Vagrant()
        for line in v.up(vm_name=machine_name, stream_output=True):
            print(line, end="")

        print("\n")
        with Connection(user=v.user(vm_name=machine_name),
                        host=v.hostname(vm_name=machine_name),
                        port=v.port(vm_name=machine_name),
                        connect_kwargs={
                            "key_filename": v.keyfile(vm_name=machine_name),
                        },) as c:

            print(f"== BEGIN vagrant andebox: {machine_name} {'=' * 80}")
            try:
                with c.cd("/vagrant"):
                    andebox_path = self.binary_path(venv, "andebox")
                    cmd = f"{andebox_path} test --venv {venv} ansible-test -R -- integration {' '.join(args.andebox_params)}"
                    if args.sudo:
                        cmd = "sudo -E " + cmd

                    c.run(cmd)
            except Exception as e:
                raise VagrantError(str(e)) from e
            finally:
                print(f"==== END vagrant andebox: {machine_name} {'=' * 80}")
